<?php

namespace Drupal\trash;

use Drupal\Component\Datetime\TimeInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Queue\QueueFactory;
use Drupal\Core\Site\Settings;

/**
 * Purges content automatically when a given time has passed.
 */
class TrashEntityPurger {

  /**
   * The name of the queue that holds local items to be purged.
   *
   * @see \Drupal\trashcan\Plugin\QueueWorker\TrashcanEntityPurgeWorker
   */
  const PURGE_QUEUE_NAME = 'trash_entity_purge';

  /**
   * Class constructor.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager.
   * @param \Drupal\trash\TrashManagerInterface $trashManager
   *   The trash manager.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The config factory service.
   * @param \Drupal\Component\Datetime\TimeInterface $time
   *   The time service.
   * @param \Drupal\Core\Queue\QueueFactory $queueFactory
   *   The queue factory.
   * @param \Drupal\Core\Site\Settings $settings
   *   The site settings.
   */
  public function __construct(
    protected EntityTypeManagerInterface $entityTypeManager,
    protected TrashManagerInterface $trashManager,
    protected ConfigFactoryInterface $configFactory,
    protected TimeInterface $time,
    protected QueueFactory $queueFactory,
    protected Settings $settings
  ) {}

  /**
   * Populate the purge queue, with items to be deleted from Drupal.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function populatePurgeQueue(string $entity_type_id): void {
    $now = $this->time->getCurrentTime();
    $config = $this->configFactory->get('trash.settings');
    $datetime = strtotime(sprintf('-%s', $config->get('auto_purge.after')), $now);

    $queue = $this->queueFactory->get(self::PURGE_QUEUE_NAME);

    $ids = $this->entityTypeManager
      ->getStorage($entity_type_id)
      ->getQuery()
      ->accessCheck(FALSE)
      ->addMetaData('trash', 'inactive')
      ->condition('deleted', $datetime, '<')
      ->execute();

    $chunks = array_chunk($ids, $this->settings->get('entity_update_batch_size'));

    foreach ($chunks as $chunk) {
      $queue->createItem([
        'batch' => $chunk,
        'entity_type_id' => $entity_type_id,
      ]);
    }
  }

  /**
   * Cron handler that will auto-purge content if enabled.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function cronPurge() {
    $config = $this->configFactory->get('trash.settings');

    if ($config->get('auto_purge.enabled')) {
      foreach ($this->trashManager->getEnabledEntityTypes() as $entityTypeId) {
        $this->populatePurgeQueue($entityTypeId);
      }
    }
  }

}
